package Data::Cache;
use strict;
use warnings;

#*******************************************************************************
# DOCUMENTATION SECTION
#*******************************************************************************

=head1 NAME

Data::Cache - Short module description

=head1 SYNOPSIS

 use Data::Cache;

=head1 DESCRIPTION

Extensive module description

=head1 AUTHOR

Jan Mach
Cesnet, z.s.p.o
jan.mach@cesnet.cz
http://www.cesnet.cz

=head1 COPYRIGHT

This program is free software; you can redistribute
it and/or modify it under the same terms as Perl itself.

The full text of the license can be found in the
LICENSE file included with this module.

=head1 SEE ALSO

perl(1), Parent::Module(3).

=head1 FUNCTION REFERENCE

=over 4

=cut

#*******************************************************************************
# LIBRARY LOADING SECTION
#*******************************************************************************

#-- Perl core modules ---------------------------------------------------------#
use Carp;
use Scalar::Util 'blessed';

#use Data::Dumper; #-+-> DEVEL ONLY <-+-#
#use Smart::Comments; #-+-> DEVEL ONLY <-+-#

#-- Perl CPAN modules ---------------------------------------------------------#

#-- Custom application modules ------------------------------------------------#

#*******************************************************************************
# CONSTANTS AND GLOBAL VARIABLES DEFINITION SECTION
#*******************************************************************************

#-- Constants -----------------------------------------------------------------#

#-- Static public class variables (our) ---------------------------------------#

#-- Static protected class variables (my) -------------------------------------#
my $SINGLETON_INSTANCE;

#*******************************************************************************
# INITIALIZATION AND CLEANUP SECTION
#*******************************************************************************

#-- Module initializations ----------------------------------------------------#
BEGIN {
    use vars qw($VERSION $DEVEL);
    $VERSION = '0.1';
    $DEVEL   = 0;
}


#-- Module clean-up code (global destructor) ----------------------------------#
END {

}

#*******************************************************************************
# IMPLEMENTATION SECTION
#*******************************************************************************

=item singleton() [PUBLIC,STATIC]

  Usage     : my $instance = Data::Cache->singleton()
  Purpose   : Create/return reference to the singleton instance
  Arguments : Same as the the new() method
  Returns   : Same as the the new() method
  Throws    : Same as the the new() method
  Comments  : Internally uses new() method
  See Also  : new() method

=cut

sub singleton
{
    my $class = shift;
    confess "Class method not invoked on class" if blessed($class);
    unless ($SINGLETON_INSTANCE) {
        $SINGLETON_INSTANCE = $class->new(@_);
    }
    return $SINGLETON_INSTANCE;
}

=item new() [PUBLIC,STATIC]

  Usage     : my $instance = Data::Cache->new()
  Purpose   : Create and return reference to the new instance
  Arguments : Same as the the _init() method
  Returns   : Data::Cache reference on success, croaks on failure
  Throws    : Dies, if not invoked on class
  Comments  : Internally uses _init() method, subclasses should never overload this method.
  See Also  : _init() method

=cut

sub new
{
    my $class = shift;
    confess "Class method not invoked on class" if blessed($class);
    ### Creating Data.Cache instance...
    my $self = bless ({}, $class);
    return $self->_init(@_);
}

=item DESTROY() [PUBLIC]

  Usage     : This method must not be used directly
  Purpose   : Class destructor
  Arguments : None
  Returns   : Nothing
  Throws    : Dies, if not invoked on instance

=cut

sub DESTROY()
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    ### Destroying Data.Cache instance...
}

=item clear(;$) [PUBLIC]

  Usage     : my $object = $instance->clear($timestamp)
  Purpose   : Clear data cache
  Arguments : INTEGER $timestamp - unix timestamp, clear everyting older [OPTIONAL]
  Returns   : Nothing
  Throws    : Dies, if not invoked on instance or in case of error

=cut

sub clear(;$)
{
    my $self = shift;
    confess "Method needs to be implemented in subclass";
}

=item fetch($;$) [PUBLIC]

  Usage     : my $object = $instance->fetch($key);
  Purpose   : Fetch data object with given key from cache
  Arguments : STRING $key - object unique key [MANDATORY]
              INTEGER $age - maximal age [OPTIONAL]
  Returns   : MIXED $object on SUCCESS, undef on FAILURE
  Throws    : Dies, if not invoked on instance or given invalid arguments

=cut

sub fetch($)
{
    my $self = shift;
    confess "Method needs to be implemented in subclass";
}

=item cache($$) [PUBLIC]

  Usage     : my $object = $instance->cache($key, $object)
  Purpose   : Store data object under given key to cache
  Arguments : STRING $key - object unique key [MANDATORY]
              MIXED $object - object to be stored
  Returns   : STRING $error on FAILURE, EMPTY STRING on SUCCESS
  Throws    : Dies, if not invoked on instance or given invalid arguments

=cut

sub cache($$)
{
    my $self = shift;
    confess "Method needs to be implemented in subclass";
}

=item stats() [PUBLIC]

  Usage     : my $stats = $cache->stats();
  Purpose   : Get cache statistics
  Arguments : NONE
  Returns   : HASH REFERENCE $stats
  Throws    : Croaks, if invoked on object

=cut

sub stats()
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    return $self->{STATS};
}

#-------------------------------------------------------------------------------
# Private interface
#-------------------------------------------------------------------------------

# _init [PROTECTED]
#
#  Usage     : return $self->_init(@_);
#  Purpose   : Initialize the new Data::Cache instance
#  Arguments : Unknown see the description in appropriate subclass
#  Returns   : Data::Cache $self reference
#  Throws    : Dies, if not invoked on instance or given invalid arguments
#  Comments  : This method must never be called directly, it is supposed to be overloaded by subclasses.

sub _init
{
    my $self = shift;
    confess "Method needs to be implemented in subclass";
}

=pod

=back

=cut

1;
