package Data::Filter::Rule::List;
use strict;
use warnings;

#*******************************************************************************
# DOCUMENTATION SECTION
#*******************************************************************************

=head1 NAME

Data::Filter::Rule::List - Mentat message filter tree list node

=head1 SYNOPSIS

 use Data::Filter::Rule::List;

=head1 DESCRIPTION



=head1 AUTHOR

 Jan Mach
 Cesnet, z.s.p.o
 jan.mach@cesnet.cz
 http://www.cesnet.cz

=head1 COPYRIGHT

This program is free software; you can redistribute
it and/or modify it under the same terms as Perl itself.

The full text of the license can be found in the
LICENSE file included with this module.


=head1 SEE ALSO

perl(1), Data::Filter::Rule(3), Data::Filter::Rule::Atom(3).

=head1 FUNCTION REFERENCE

=over 4

=cut

#*******************************************************************************
# LIBRARY LOADING SECTION
#*******************************************************************************

#-- Perl core modules ---------------------------------------------------------#
use Carp;
use Scalar::Util 'blessed';

#use Data::Dumper; #--- DEVELOPMENT ONLY ---#
#use Smart::Comments; #--- DEVELOPMENT ONLY ---#

#-- Perl CPAN modules ---------------------------------------------------------#

#-- Custom application modules ------------------------------------------------#
use Data::Filter::Rule::Atom;
use Data::Filter::Rule::Constant;
use Data::Core qw(:c_dtypes);

#*******************************************************************************
# CONSTANTS AND GLOBAL VARIABLES DEFINITION SECTION
#*******************************************************************************

#-- Constants -----------------------------------------------------------------#

#-- Static public class variables (our) ---------------------------------------#

#-- Static protected class variables (my) -------------------------------------#

#*******************************************************************************
# INITIALIZATION AND CLEANUP SECTION
#*******************************************************************************

#-- Module initializations ----------------------------------------------------#
BEGIN {
    use vars qw($VERSION $DEVEL @ISA);
    $VERSION = '0.1';
    $DEVEL   = 0;
    @ISA     = ('Data::Filter::Rule::Atom');
}


#-- Module clean-up code (global destructor) ----------------------------------#
END {

}

#*******************************************************************************
# IMPLEMENTATION SECTION
#*******************************************************************************

=item datatype(;$) [PUBLIC]

  Usage     : my $result = $rule->datatype();
  Purpose   :
  Returns   :
  Arguments :
  Throws    :
  Comments  :
  See Also  :

=cut

sub datatype(;$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);

    return $self->{DATATYPE};
}

=item value($) [PUBLIC]

  Usage     : my $result = $rule->value();
  Purpose   :
  Returns   :
  Arguments :
  Throws    :
  Comments  :
  See Also  :

=cut

sub value()
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);

    return [ map { (ref $_)?$_->to_numeric():$_; } @{$self->{VALUES}}]
}

=item value_raw($) [PUBLIC]

  Usage     : my $result = $rule->value_raw();
  Purpose   :
  Returns   :
  Arguments :
  Throws    :
  Comments  :
  See Also  :

=cut

sub value_raw()
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);

    return $self->{VALUES};
}

#-------------------------------------------------------------------------------

=item compile($$) [PUBLIC]

  Usage     : my $result = Data::Filter::Rule->compile();
  Purpose   :
  Returns   :
  Arguments :
  Throws    :
  Comments  :
  See Also  :

=cut

sub compile($$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($datatype,) = @_;
    confess "You have to specify the datatype for which to compile the list values" unless $datatype;

    if ($self->{DATATYPE} eq DT_UNKNOWN) {
        $self->{VALUES} = [
            map { Data::Filter::Rule::Constant->compile_value($_, $datatype) } @{$self->{VALUES}}
        ];
        $self->{DATATYPE} = $datatype;
    }

    return $self;
}

#-------------------------------------------------------------------------------

=item to_string() [PUBLIC]

  Usage     : my $result = Data::Filter::Rule->to_string();
  Purpose   :
  Returns   :
  Arguments :
  Throws    :
  Comments  :
  See Also  :

=cut

sub to_string()
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);

    return '['.join(',', map { '"'.$_.'"'} @{$self->value_raw()}).']';
}

#-------------------------------------------------------------------------------
# Private interface
#-------------------------------------------------------------------------------

# _init($$;$) [PROTECTED]
#
#  Usage     :
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Croaks, if invoked on class
#  Comments  :
#  See Also  :

sub _init($$;$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($ruletype, $values, $datatype) = @_;
    confess "List values must be given as argument" unless defined($values);

    $self->{DATATYPE} = DT_UNKNOWN;

    if (ref $values eq 'ARRAY') {
        $self->{VALUES} = $values;
    }
    else {
        $self->{VALUES} = [split(/[,;]+/, $values)];
        $self->{VALUES} = [map { $_ =~ s/^\s+//; $_ =~ s/\s+$//; $_ =~ s/^"//; $_ =~ s/"$//; $_; } @{$self->{VALUES}}];
    }

    $self->compile($datatype) if $datatype;

    return $self;
}

=pod

=back

=cut

1;
