package Data::Filter::Traverser;
use strict;
use warnings;


#*******************************************************************************
# DOCUMENTATION SECTION
#*******************************************************************************

=head1 NAME

Data::Filter::Traverser - Base class for all objects capable of traversing through filter tree

=head1 SYNOPSIS

 use Data::Filter::Traverser;

=head1 DESCRIPTION



=head1 AUTHOR

 Jan Mach
 Cesnet, z.s.p.o
 jan.mach@cesnet.cz
 http://www.cesnet.cz

=head1 COPYRIGHT

This program is free software; you can redistribute
it and/or modify it under the same terms as Perl itself.

The full text of the license can be found in the
LICENSE file included with this module.

=head1 SEE ALSO

perl(1).

=head1 FUNCTION REFERENCE

=over 4

=cut

#*******************************************************************************
# LIBRARY LOADING SECTION
#*******************************************************************************

#-- Perl core modules ---------------------------------------------------------#
use Carp;
use Scalar::Util 'blessed';

use Data::Dumper; #--- DEVELOPMENT ONLY ---#
#use Smart::Comments; #--- DEVELOPMENT ONLY ---#

#-- Perl CPAN modules ---------------------------------------------------------#

#-- Custom application modules ------------------------------------------------#
use Data::Core qw(:c_dtypes);

#*******************************************************************************
# CONSTANTS AND GLOBAL VARIABLES DEFINITION SECTION
#*******************************************************************************

#-- Constants -----------------------------------------------------------------#
use constant OP_UNARY  => 'UNARY';
use constant OP_BINARY => 'BINARY';

use constant OP_LOGIC => 'LOGICAL';
use constant OP_COMP  => 'COMPARISON';
use constant OP_CALC  => 'CALCULATION';

#-- Static public class variables (our) ---------------------------------------#

#-- Static protected class variables (my) -------------------------------------#

# Meta structure describing available operations
my @OPERATIONS_META = (
        ['not', OP_UNARY, OP_LOGIC,
         sub { my ($l,) = @_; return 1 unless $l; return 0; }],
        ['and', OP_BINARY, OP_LOGIC,
         sub { my ($l, $r) = @_; return 1 if ($l and $r); return 0; }],
        ['or', OP_BINARY, OP_LOGIC,
         sub { my ($l, $r) = @_; return 1 if ($l or $r); return 0; }],
        ['xor', OP_BINARY, OP_LOGIC,
         sub { my ($l, $r) = @_; return 1 if ($l and not $r); return 1 if ($r and not $l); return 0; }],

        ['eq_string', OP_BINARY, OP_COMP,
         sub { my ($l, $r) = @_; return 0 unless (defined $l and defined $r); return 1 if ($l eq $r); return 0; }],
        ['eq_numeric', OP_BINARY, OP_COMP,
         sub { my ($l, $r) = @_; return 0 unless (defined $l and defined $r); return 1 if ($l == $r); return 0; }],
        ['ne_string', OP_BINARY, OP_COMP,
         sub { my ($l, $r) = @_; return 0 unless (defined $l and defined $r); return 1 if ($l ne $r); return 0; }],
        ['ne_numeric', OP_BINARY, OP_COMP,
         sub { my ($l, $r) = @_; return 0 unless (defined $l and defined $r); return 1 if ($l != $r); return 0; }],
        ['gt_string', OP_BINARY, OP_COMP,
         sub { my ($l, $r) = @_; return 0 unless (defined $l and defined $r); return 1 if ($l gt $r); return 0; }],
        ['gt_numeric', OP_BINARY, OP_COMP,
         sub { my ($l, $r) = @_; return 0 unless (defined $l and defined $r); return 1 if ($l >  $r); return 0; }],
        ['lt_string', OP_BINARY, OP_COMP,
         sub { my ($l, $r) = @_; return 0 unless (defined $l and defined $r); return 1 if ($l lt $r); return 0; }],
        ['lt_numeric', OP_BINARY, OP_COMP,
         sub { my ($l, $r) = @_; return 0 unless (defined $l and defined $r); return 1 if ($l <  $r); return 0; }],
        ['ge_string', OP_BINARY, OP_COMP,
         sub { my ($l, $r) = @_; return 0 unless (defined $l and defined $r); return 1 if ($l ge $r); return 0; }],
        ['ge_numeric', OP_BINARY, OP_COMP,
         sub { my ($l, $r) = @_; return 0 unless (defined $l and defined $r); return 1 if ($l >= $r); return 0; }],
        ['le_string', OP_BINARY, OP_COMP,
         sub { my ($l, $r) = @_; return 0 unless (defined $l and defined $r); return 1 if ($l le $r); return 0; }],
        ['le_numeric', OP_BINARY, OP_COMP,
         sub { my ($l, $r) = @_; return 0 unless (defined $l and defined $r); return 1 if ($l <= $r); return 0; }],
        ['in_numeric', OP_BINARY, OP_COMP,
         sub { my ($l, $r) = @_; return 0 unless (defined $l and defined $r); return 1 if ($l == $r); return 0; }],
        ['in_string', OP_BINARY, OP_COMP,
         sub { my ($l, $r) = @_; return 0 unless (defined $l and defined $r); return 1 if ($l eq $r); return 0; }],
        ['like_numeric', OP_BINARY, OP_COMP,
         sub { my ($l, $r) = @_; return 0 unless (defined $l and defined $r); return 1 if ("$l" =~ /$r/); return 0; }],
        ['like_string', OP_BINARY, OP_COMP,
         sub { my ($l, $r) = @_; return 0 unless (defined $l and defined $r); return 1 if ("$l" =~ /$r/); return 0; }],

        ['add', OP_BINARY, OP_CALC,
         sub { my ($l, $r) = @_; return ($l + $r); }],
        ['sub', OP_BINARY, OP_CALC,
         sub { my ($l, $r) = @_; return ($l - $r); }],
        ['mul', OP_BINARY, OP_CALC,
         sub { my ($l, $r) = @_; return ($l * $r); }],
        ['div', OP_BINARY, OP_CALC,
         sub { my ($l, $r) = @_; return ($l / $r); }],
        ['mod', OP_BINARY, OP_CALC,
         sub { my ($l, $r) = @_; return ($l % $r); }],
        ['con', OP_BINARY, OP_CALC,
         sub { my ($l, $r) = @_; return ("$l" . "$r"); }],
        ['rep', OP_BINARY, OP_CALC,
         sub { my ($l, $r) = @_; return ("$l" x int($r)); }],

    );

# Operation type mapping hashes
my %OPERATIONS = ();        # All operations and associated callbacks
my %OPERATIONS_CRD = ();    # Operation cardinality
my %OPERATIONS_TPE = ();    # Operation type
my %OPERATIONS_UNR = ();    # Unary operations
my %OPERATIONS_BIN = ();    # Binary operations
my %OPERATIONS_LOG = ();    # Logical operations
my %OPERATIONS_CMP = ();    # Comparison operations
my %OPERATIONS_CAL = ();    # Calculation operations
my %OPERATIONS_DTD = ();    # Data type dependent operations
my $t;
map {
    $t = $_->[0]; $t =~ s/_string|_numeric//;

    $OPERATIONS{$_->[0]} = $_->[3];
    $OPERATIONS_CRD{$_->[0]} = $_->[1];
    $OPERATIONS_TPE{$_->[0]} = $_->[2];
    $OPERATIONS_UNR{$_->[0]} = 1 if $_->[1] eq OP_UNARY;
    $OPERATIONS_BIN{$_->[0]} = 1 if $_->[1] eq OP_BINARY;
    $OPERATIONS_LOG{$_->[0]} = 1 if $_->[2] eq OP_LOGIC;
    $OPERATIONS_CMP{$_->[0]} = 1 if $_->[2] eq OP_COMP;
    $OPERATIONS_CAL{$_->[0]} = 1 if $_->[2] eq OP_CALC;
    $OPERATIONS_DTD{$t} = 1 if $_->[2] eq OP_COMP;
} @OPERATIONS_META;

#*******************************************************************************
# INITIALIZATION AND CLEANUP SECTION
#*******************************************************************************

#-- Module initializations ----------------------------------------------------#
BEGIN {
    use vars qw($VERSION $DEVEL);
    $VERSION = '0.1';
    $DEVEL   = 0;
}


#-- Module clean-up code (global destructor) ----------------------------------#
END {

}

#*******************************************************************************
# IMPLEMENTATION SECTION
#*******************************************************************************

=item new() [PUBLIC,STATIC]

  Usage     : my $instance = Data::Filter::Traverser->new()
  Purpose   : Create and return reference to the new instance
  Arguments : Same as the the _init() method
  Returns   : Data::Filter::Traverser reference on success, croaks on failure
  Throws    : Croaks, if invoked on object
  Comments  : Internally uses _init() method, subclasses should never overload this method.
  See Also  : _init() method

=cut

sub new
{
    my $class = shift;
    confess "Class method not invoked on class" if blessed($class);
    my %args = @_;

    my $self = bless ({}, $class);

    # Setter mode
    if ($args{dt_resolver}) {
        confess "Invalid variable datatype resolver" unless blessed($args{dt_resolver}) and $args{dt_resolver}->can('path_datatype');
        $self->{DT_RESOLVER} = $args{dt_resolver};
    }

    # Let subclasses perform specific initializations
    return $self->_init(@_);
}

#-------------------------------------------------------------------------------

=item FILTER CALLBACKS [PUBLIC,ABSTRACT]

  Usage     : return $traverser->$name(@args);
  Purpose   : Rule tree traversal callbacks, for each rule class one callback method
  Arguments : Depend on particular callback
  Returns   : Any result from callback will be passed to the parent node callback as argument
  Throws    : Croaks, if invoked on object
  Comments  : These methods are called from Data::Filter::traverse method
              during filter tree traversal. Arguments are dependent on

=cut

sub constant($;@)
{
    my $self = shift;
    confess "Method needs implementation";
    my ($filter, @args) = @_;
}
sub list($;@)
{
    my $self = shift;
    confess "Method needs implementation";
    my ($filter, @args) = @_;
}
sub existence($;@)
{
    my $self = shift;
    confess "Method needs implementation";
    my ($filter, @args) = @_;
}
sub variable($;@)
{
    my $self = shift;
    confess "Method needs implementation";
    my ($filter, @args) = @_;
}
# ---
sub operationUnary($$$$;@)
{
    my $self = shift;
    confess "Method needs implementation";
    my ($filter, $operation, $subfilter, $result, @args) = @_;
}
sub operationBinary($$$$$$;@)
{
    my $self = shift;
    confess "Method needs implementation";
    my ($filter, $operation, $l_subfilter, $r_subfilter, $l_result, $r_result, @args) = @_;
}

#-------------------------------------------------------------------------------

=item evaluate_bin($$$) [PUBLIC,HYBRID]

  Usage     : my $result = $instance->evaluate_bin($evaluator, $left, $right);
  Purpose   : Evaluate two values using given binary evaluator
  Arguments : STRING $evaluator - name of the evaluator to use
              MIXED $left - left value
              MIXED $right - right value
  Returns   : MIXED evaluation result

=cut

sub evaluate_bin($$$)
{
    my $invocant = shift;
    my ($ev, $left, $right) = @_;
    confess "Evaluator must be given as argument" unless defined($ev);
    confess "Invalid evaluator '$ev', not a binary operation" unless exists $OPERATIONS_BIN{$ev};
    #confess "Left and right operands and evaluator must be given as arguments" unless (defined($left) and defined($right));

    return &{$OPERATIONS{$ev}}($left, $right);
}

=item evaluate_unr($$$) [PUBLIC,HYBRID]

  Usage     : my $result = $instance->evaluate_unr($evaluator, $left);
  Purpose   : Evaluate one value using given unary evaluator
  Arguments : STRING $evaluator - name of the evaluator to use
              MIXED $left - left value
  Returns   : MIXED evaluation result

=cut

sub evaluate_unr($$$)
{
    my $invocant = shift;
    my ($ev, $left) = @_;
    confess "Evaluator must be given as argument" unless defined($ev);
    confess "Invalid evaluator '$ev', not an unary operation" unless exists $OPERATIONS_UNR{$ev};
    #confess "Left operand must be given as argument" unless defined($left);

    return &{$OPERATIONS{$ev}}($left);
}

=item evaluate($$$) [PUBLIC,HYBRID]

  Usage     : my $result = $instance->evaluate($evaluator, @args);
  Purpose   : Evaluate given values using given evaluator
  Arguments : STRING $evaluator - name of the evaluator to use
              MIXED @args - appropriate number of arguments (one for unary, two for binary operation)
  Returns   : MIXED evaluation result

=cut

sub evaluate($$$)
{
    my $invocant = shift;
    my ($ev, @args) = @_;
    confess "Evaluator must be given as argument" unless defined($ev);
    confess "Invalid evaluator '$ev'" unless exists $OPERATIONS{$ev};

    return $invocant->evaluate_bin($ev, @args) if ($OPERATIONS_BIN{$ev});
    return $invocant->evaluate_unr($ev, @args) if ($OPERATIONS_UNR{$ev});
    confess "Invalid evaluator '$ev', not a binary or unary operation";
}

#-------------------------------------------------------------------------------

=item evaluate_n_bin($$$) [PUBLIC,HYBRID]

  Usage     : my $result = $instance->evaluate_n_bin($evaluator, $left, $right);
  Purpose   : Evaluate n left values with n right values using given binary evaluator
  Arguments : STRING $evaluator - name of the evaluator to use
              ARRAY REFERENCE $left - left values
              ARRAY REFERENCE $right - right values
  Returns   : MIXED evaluation result

=cut

sub evaluate_n_bin($$$)
{
    my $invocant = shift;
    my ($ev, $left, $right) = @_;
    confess "Evaluator must be given as argument" unless defined($ev);
    confess "Invalid evaluator '$ev', not a binary operation" unless exists $OPERATIONS_BIN{$ev};
    confess "Left and right operands must be given as arguments" unless (defined($left) and defined($right));

    # Transform the operands to the array, if necessary
    $left  = [$left]  unless ref $left  eq 'ARRAY';
    $right = [$right] unless ref $right eq 'ARRAY';

    return $invocant->_eval_n_bin_bool($ev, $left, $right) if ($OPERATIONS_CMP{$ev} or $OPERATIONS_LOG{$ev});
    return $invocant->_eval_n_bin_mixed($ev, $left, $right);
}

=item evaluate_n_unr($$$) [PUBLIC,HYBRID]

  Usage     : my $result = $instance->evaluate_n_unr($evaluator, $left);
  Purpose   : Evaluate n left values using given unary evaluator
  Arguments : STRING $evaluator - name of the evaluator to use
              ARRAY REFERENCE $left - left values
  Returns   : MIXED evaluation result

=cut

sub evaluate_n_unr($$$)
{
    my $invocant = shift;
    my ($ev, $left) = @_;
    confess "Evaluator must be given as argument" unless defined($ev);
    confess "Invalid evaluator '$ev', not an unary operation" unless exists $OPERATIONS_UNR{$ev};
    confess "Left operand must be given as argument" unless defined($left);

    # Transform the operand to the array, if necessary
    $left = [$left] unless ref $left eq 'ARRAY';

    return $invocant->_eval_n_unr_bool($ev, $left);
}

=item evaluate_n($$$) [PUBLIC,HYBRID]

  Usage     : my $result = $instance->evaluate_n($evaluator, $left, $right);
  Purpose   : Evaluate n left values with n right values using given evaluator
  Arguments : STRING $evaluator - name of the evaluator to use
              ARRAY REFERENCE $left - left values
              ARRAY REFERENCE $right - right values
  Returns   : MIXED evaluation result

=cut

sub evaluate_n($$$)
{
    my $invocant = shift;
    my ($ev, @args) = @_;
    confess "Evaluator must be given as argument" unless defined($ev);
    confess "Invalid evaluator '$ev'" unless exists $OPERATIONS{$ev};

    return $invocant->evaluate_n_bin($ev, @args) if ($OPERATIONS_BIN{$ev});
    return $invocant->evaluate_n_unr($ev, @args);
}

#-------------------------------------------------------------------------------

=item operation_cardinality($) [PUBLIC,HYBRID]

  Usage     : my $result = $instance->operation_cardinality($operation);
  Purpose   : Determine the operation cardinality
  Arguments : STRING $operation - name of the operation
  Returns   : STRING operation cardinality

=cut

sub operation_cardinality($)
{
    my $invocant = shift;
    my ($operation,) = @_;
    confess "Operation must be given as argument" unless defined($operation);

    $operation .= '_string' if ($OPERATIONS_DTD{$operation});
    return $OPERATIONS_CRD{$operation};
}

=item operation_type($) [PUBLIC,HYBRID]

  Usage     : my $result = $instance->operation_type($operation);
  Purpose   : Determine the operation type
  Arguments : STRING $operation - name of the operation
  Returns   : STRING operation type

=cut

sub operation_type($)
{
    my $invocant = shift;
    my ($operation,) = @_;
    confess "Operation must be given as argument" unless defined($operation);

    $operation .= '_string' if ($OPERATIONS_DTD{$operation});
    return $OPERATIONS_TPE{$operation};
}

#-------------------------------------------------------------------------------

=item pick_evaluator($$$) [PUBLIC,HYBRID]

  Usage     : my $evaluator = $instance->pick_evaluator($operation, $left, $right);
  Purpose   : Choose the appropriate evaluator according to the given operation and argument datatypes
  Arguments : STRING $operation - name of the compare operation
              STRING $left - left operand
              STRING $right - right operand
  Returns   : STRING $evaluator name of the appropriate evaluator

=cut

sub pick_evaluator($$$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($operation, $left, $right) = @_;
    confess "Operation name and left operand must be given as arguments" unless (defined($operation) and defined($left));
    $operation = lc($operation);

    # Unless operation is comparison, evaluator has the same name as operation
    return $operation unless $OPERATIONS_DTD{$operation};

    # Determine the primary datatypes of the operands
    my $dl = $left->datatype();
    my $dr = $right->datatype();
    print STDERR "Datatype a-left:  $dl\n" if $DEVEL;
    print STDERR "Datatype a-right: $dr\n" if $DEVEL;
    # Determine the real datatypes of the variable operands
    if ($dl eq DT_VARIABLE) { $dl = $self->resolve_datatype($left); }
    if ($dr eq DT_VARIABLE) { $dr = $self->resolve_datatype($right); }
    print STDERR "Datatype b-left:  $dl\n" if $DEVEL;
    print STDERR "Datatype b-right: $dr\n" if $DEVEL;
    
    if ("${dl}-${dr}" =~ /(INT|REAL|ADDRESS|IPV4|IPV6|DATETIME)-(UNKNOWN|INT|REAL|ADDRESS|IPV4|IPV6|DATETIME )|(UNKNOWN|INT|REAL|ADDRESS|IPV4|IPV6|DATETIME)-(INT|REAL|ADDRESS|IPV4|IPV6|DATETIME)/) {
        return lc("${operation}_numeric");
    }
    else {
        return lc("${operation}_string");
    }
}

#-------------------------------------------------------------------------------

=item resolve_datatype($) [PUBLIC]

  Usage     :
  Purpose   :
  Returns   :
  Arguments :
  Throws    :
  Comments  :
  See Also  :

=cut

sub resolve_datatype($)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($filter,) = @_;
    confess "Path datatype resolver was not set up using 'dt_resolver' configuration during object instantination" unless $self->{DT_RESOLVER};

    return $self->{DT_RESOLVER}->path_datatype($filter->value());
}

#-------------------------------------------------------------------------------
# Private interface
#-------------------------------------------------------------------------------

# _init [PROTECTED]
#
#  Usage     : return $self->_init(@_);
#  Purpose   : Initialize the new Data::Filter::Traverser instance
#  Arguments :
#  Returns   : Data::Filter::Traverser reference
#  Throws    : Croaks, if invoked on class
#  Comments  : This method must never be called directly, it is supposed to be overloaded by subclasses.

sub _init
{
    my $self = shift;
    confess "Method needs implementation";
}

# _eval_n_bin_bool($$$) [PROTECTED,HYBRID]
#
#  Usage     :
#  Purpose   :
#  Arguments :
#  Returns   :

sub _eval_n_bin_bool($$$)
{
    my $invocant = shift;
    my ($ev, $left, $right) = @_;

    # Perform evaluations n to n
    foreach my $v1 (@$left) {
        foreach my $v2 (@$right) {
            return 1 if $invocant->evaluate_bin($ev, $v1, $v2);
        }
    }
    return 0;
}

# _eval_n_bin_mixed($$$) [PROTECTED,HYBRID]
#
#  Usage     :
#  Purpose   :
#  Arguments :
#  Returns   :

sub _eval_n_bin_mixed($$$)
{
    my $invocant = shift;
    my ($ev, $left, $right) = @_;

    # Get the lengths of the operands
    my $ll = scalar (@$left);
    my $lr = scalar (@$right);

    # Storage for result
    my $result = [];

    # If we want to calculate n to n values, they must have same legth
    if ($ll > 1 and $lr > 1) {
        confess "Evaluation operands must have same length" unless $ll == $lr;
        for (my $i = 0; $i < $ll; $i++) {
            push(@$result, $invocant->evaluate_bin($ev, $left->[$i], $right->[$i]));
        }
    }
    # Now either left, or right is just single value
    else {
        # We do not know which one and do not care, perhaps optimization is needed?
        foreach my $v1 (@$left) {
            foreach my $v2 (@$right) {
                push(@$result, $invocant->evaluate_bin($ev, $v1, $v2));
            }
        }
    }

    return $result;
}

# _eval_n_unr_bool($$) [PROTECTED,HYBRID]
#
#  Usage     :
#  Purpose   :
#  Arguments :
#  Returns   :

sub _eval_n_unr_bool($$)
{
    my $invocant = shift;
    my ($ev, $left) = @_;

    # Perform evaluations n
    foreach my $v1 (@$left) {
        return 1 if $invocant->evaluate_unr($ev, $v1);
    }
    return 0;
}

=pod

=back

=cut

1;
