package Hawat::Module::Log;
use strict;
use warnings;

#*******************************************************************************
# DOCUMENTATION SECTION
#*******************************************************************************

=head1 NAME

Hawat::Module::Log - Short module description

=head1 SYNOPSIS

 use Hawat::Module::Log;

=head1 DESCRIPTION

Extensive module description

=head1 AUTHOR

 Jan Mach
 Cesnet, z.s.p.o
 jan.mach@cesnet.cz
 http://www.cesnet.cz

=head1 COPYRIGHT

This program is free software; you can redistribute
it and/or modify it under the same terms as Perl itself.

The full text of the license can be found in the
LICENSE file included with this module.

=head1 SEE ALSO

perl(1).

=head1 FUNCTION REFERENCE

=over 4

=cut

#*******************************************************************************
# LIBRARY LOADING SECTION
#*******************************************************************************

#-- Perl core modules ---------------------------------------------------------#
use Carp;
use Scalar::Util 'blessed';
use Time::HiRes qw (time);
use File::Find;

#use Data::Dumper; #-+-> DEVEL ONLY <-+-#
#use Smart::Comments; #-+-> DEVEL ONLY <-+-#

#-- Perl CPAN modules ---------------------------------------------------------#

#-- Custom application modules ------------------------------------------------#
use Hawat::Module;

#*******************************************************************************
# CONSTANTS AND GLOBAL VARIABLES DEFINITION SECTION
#*******************************************************************************

#-- Constants -----------------------------------------------------------------#

#-- Static public class variables (our) ---------------------------------------#

#-- Static protected class variables (my) -------------------------------------#

# Hash containing all valid commands supported by this module and their asociated
# callback names for indirect execution.
my %COMMANDS = (
            'clear' => {
                    '_DESC_' => 'Clear Mentat component log files',
                    ''      => [
                            '_command_clear_all' ,
                            'Clear all Mentat component log files present on this system (same as "all", see it for more info)',
                        ],
                    'all'   => [
                            '_command_clear_all' ,
                            'Clear all Mentat component log files present on this system',
                        ],
                    'only'  => [
                            '_command_clear_only',
                            'Clear log files of selected Mentat components',
                            '(STRING) pattern - PCRE pattern to apply on log file name [MANDATORY]',
                        ],
                },
            'list'  => {
                    '_DESC_' => 'Find and list Mentat component log files',
                    ''      => [
                            '_command_list_all' ,
                            'Find and list all Mentat component log files present on this system (same as "all", see it for more info)',
                        ],
                    'all'   => [
                            '_command_list_all' ,
                            'Find and list all Mentat component log files present on this system',
                        ],
                    'only'  => [
                            '_command_list_only',
                            'Find and list log files of selected Mentat components',
                            '(STRING) pattern - PCRE pattern to apply on log file name [MANDATORY]',
                        ],
                },
        );

#*******************************************************************************
# INITIALIZATION AND CLEANUP SECTION
#*******************************************************************************

#-- Module initializations ----------------------------------------------------#
BEGIN {
    use vars qw($VERSION $DEVEL @ISA);
    $VERSION = '0.1';
    $DEVEL   = 0;
    @ISA     = ('Hawat::Module');
}


#-- Module clean-up code (global destructor) ----------------------------------#
END {

}

#*******************************************************************************
# IMPLEMENTATION SECTION
#*******************************************************************************

=item get_commands() [PUBLIC]

  Usage     :
  Purpose   : Return list of all commands this module provides
  Arguments :
  Returns   :
  Throws    : Croaks, if invoked on class
  Comments  :
  See Also  :

=cut

sub get_commands()
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    return %COMMANDS;
}

#-------------------------------------------------------------------------------
# Private interface
#-------------------------------------------------------------------------------

# _init [PROTECTED]
#
#  Usage     : return $self->_init(@_);
#  Purpose   : Initialize the new Hawat::Module::Log instance
#  Arguments :
#  Returns   :
#  Throws    : Croaks, if invoked on class
#  Comments  : This method must never be called directly, it is supposed to be overloaded by subclasses.
#  See Also  :

sub _init
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);

    $self->{LOG_DIRS}   = ['/var/mentat/log/', '/opt/mentat/var/log/'];
    $self->{LOG_FILES}  = {};

    return $self;
}

# _add_log_to_list [PROTECTED]
#
#  Usage     :
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Croaks, if invoked on class
#  Comments  :
#  See Also  :

sub _add_log_to_list
{
    my $self    = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);

    # Skip directories
    return unless -f $File::Find::name;

    # Skip everything that is not a mentat system log file
    return unless $_ =~ /^[-\d\w]+\.log/;

    # Apply optional log file pattern
    if ($self->{PATTERN}) {
        my $pattern = $self->{PATTERN};
        return unless $_ =~ /$pattern/;
    }

    # Mark the file for adding to the package
    $self->{LOG_FILES}->{$File::Find::name} = $_;
}


# _find_log_files(;$) [PROTECTED]
#
#  Usage     :
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Croaks, if invoked on class
#  Comments  :
#  See Also  :

sub _find_log_files(;$)
{
    my $self    = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($pattern,) = @_;

    $self->{PATTERN} = $pattern || undef;

    foreach my $dir (@{$self->{LOG_DIRS}}) {
        find({'wanted' => sub { $self->_add_log_to_list }, 'untaint' => 1}, $dir) if -d $dir;
    }
    return $self->{LOG_FILES};
}

#-------------------------------------------------------------------------------

# _clear_log_files(;$) [PROTECTED]
#
#  Usage     :
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Croaks, if invoked on class
#  Comments  :
#  See Also  :

sub _clear_log_files(;$)
{
    my $self    = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($pattern,) = @_;

    my $start = time();

    my $files = $self->_find_log_files($pattern);
    my @file_names = sort keys(%$files);

    $self->res->{'count'} = scalar(@file_names);
    foreach my $file (@file_names) {
        $self->out("   - $file");
        if ($file =~ /mentat-controller/) {
            my $r = unlink $file;
            if ($r) {
                $self->out(" [REMOVED]\n", 'bold green');
            }
            else {
                $self->out(" [FAILURE]\n", 'bold red');
            }
        }
        elsif ($file =~ /\.log$/) {
            if(system('/bin/echo -n "" > ' . "$file") == 0) {
                $self->out(" [SUCCESS]\n", 'bold green');
                push(@{$self->res->{'success'}}, $file);
            }
            else {
                $self->out(" [FAILURE]\n", 'bold red');
                push(@{$self->res->{'failure'}}, $file);
            }
        }
        else {
            my $r = unlink $file;
            if ($r) {
                $self->out(" [REMOVED]\n", 'bold green');
            }
            else {
                $self->out(" [FAILURE]\n", 'bold red');
            }
        }
    }

    my $stop = time();
    $self->out(" - Log clearing duration: " . ($stop - $start) . " [s]\n");
    $self->res->{'duration'} = $stop - $start;
}

# _list_log_files(;$) [PROTECTED]
#
#  Usage     :
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Croaks, if invoked on class
#  Comments  :
#  See Also  :

sub _list_log_files(;$)
{
    my $self    = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($pattern,) = @_;

    my $start = time();

    my $files = $self->_find_log_files($pattern);
    my @file_names = sort keys(%$files);

    $self->res->{'count'} = scalar(@file_names);
    foreach my $file (@file_names) {
        $self->out("   - $file\n");
    }

    my $stop = time();
    $self->out(" - Log files listing duration: " . ($stop - $start) . " [s]\n");
    $self->res->{'duration'} = $stop - $start;
}

#-----------------------------------------------------------------------

# _command_clear_all [PROTECTED]
#
#  Usage     :
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Croaks, if invoked on class
#  Comments  :
#  See Also  :

sub _command_clear_all
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($chain, @args) = @_;
    die "Invalid usage: [$0 @$chain] command does not accept any arguments\n" if @args;

    $self->out(" - Clearing all Mentat log files:\n");
    $self->_clear_log_files();
}

# _command_clear_only [PROTECTED]
#
#  Usage     :
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Croaks, if invoked on class
#  Comments  :
#  See Also  :

sub _command_clear_only
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($chain, @args) = @_;
    my ($pattern,) = @args;
    die "Invalid usage: [$0 @$chain] command must receive log file name pattern as argument\n" unless $pattern;

    $self->out(" - Clearing Mentat log files according to the pattern '$pattern':\n");
    $self->res->{'pattern'} = $pattern;
    $self->_clear_log_files($pattern);
}

#-----------------------------------------------------------------------

# _command_list_all [PROTECTED]
#
#  Usage     :
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Croaks, if invoked on class
#  Comments  :
#  See Also  :

sub _command_list_all
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($chain, @args) = @_;
    die "Invalid usage: [$0 @$chain] command does not accept any arguments\n" if @args;

    $self->out(" - Listing all Mentat log files:\n");
    $self->_list_log_files();
}

# _command_list_only [PROTECTED]
#
#  Usage     :
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Croaks, if invoked on class
#  Comments  :
#  See Also  :

sub _command_list_only
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($chain, @args) = @_;
    my ($pattern,) = @args;
    die "Invalid usage: [$0 @$chain] command must receive log file name pattern as argument\n" unless $pattern;

    $self->out(" - Listing Mentat log files according to the pattern '$pattern':\n");
    $self->res->{'pattern'} = $pattern;
    $self->_list_log_files($pattern);
}

=pod

=back

=cut

1;
