package Mentat::Processor::Engine::Filter;
use strict;
use warnings;

#*******************************************************************************
# DOCUMENTATION SECTION
#*******************************************************************************

=head1 NAME

Mentat::Processor::Engine::Filter - Engine performing ALERT message filtering

=head1 SYNOPSIS

 use Mentat::Processor::Engine::Filter;

 # During the instantination of the engine pass the name of the file containing the filter grammmar
 # and name of the file containing filtering rules as arguments
 my $engine = Mentat::Processor::Engine::Filter->new($engine, $filter_grammar_file, $filter_rules_file);

=head1 DESCRIPTION

Filtering algorithm of this COG is inspired by Apache webserver. There are two
chains (or groups) of filtering rules, 'allow' and 'deny'. 'allow' rules describe what
messages may pass, 'deny' rules describe messages, that will be rejected. Each chain
may have multiple filter rules defined and they are processed in order of appearance
in config file and logically ORed together. If the message matches a 'allow' rule
in the chain, it is immediatelly accepted and further processing is skipped. If
the message matches a 'deny' rule in the chain, it is immediatelly rejected and
further processing is skipped. If the message did not matched any rule in any chain,
the result will be the defined default policy ('allow' or 'deny'). The order of
processing of the chains as a whole, if the 'allow' or 'deny' rules will be processed
first, is configurable option.

=head1 FILTER RULES CONFIGURATION FILE SYNTAX

 # Order in which to process the filter chains
 # [DEFAULT='allow']
 order = allow, deny
 order = allow
 order = deny, allow
 order = deny

 # Default action if the message did not matched any rule in any filter chain
 # [DEFAULT='deny']
 default_action = deny
 default_action = allow

 # Logical operation to be applied to rules in a 'allow' chain
 # [DEFAULT='or']
 logop_allow = and
 logop_allow = or

 # Logical operation to be applied to rules in a 'deny' chain
 # [DEFAULT='or']
 logop_deny = and
 logop_deny = or

 # Rules populating the 'allow' chain (processed in order of appearance)
 allow = ? Alert
 allow = Alert/@messageid == "1234" and Alert/Target/@ident eq "d1c2b3a4"

 # Rules populating the 'deny' chain (processed in order of appearance)
 deny = ? Heartbeat
 deny = Alert/@messageid == "1234" and Alert/Target/@ident eq "d1c2b3a4"

=head1 AUTHOR

 Jan Mach
 Cesnet, z.s.p.o
 jan.mach@cesnet.cz
 http://www.cesnet.cz

=head1 COPYRIGHT

This program is free software; you can redistribute
it and/or modify it under the same terms as Perl itself.

The full text of the license can be found in the
LICENSE file included with this module.

=head1 SEE ALSO

perl(1).

=head1 FUNCTION REFERENCE

=over 4

=cut

#*******************************************************************************
# LIBRARY LOADING SECTION
#*******************************************************************************

#-- Perl core modules ---------------------------------------------------------#
use Carp;
use Scalar::Util 'blessed';
use Time::HiRes qw(time);

#use Data::Dumper; #-+-> DEVEL ONLY <-+-#
#use Smart::Comments; #-+-> DEVEL ONLY <-+-#

#-- Perl CPAN modules ---------------------------------------------------------#

#-- Custom application modules ------------------------------------------------#
use Mentat::Core qw(:c_general);
use Mentat::Processor::Engine;
use Data::Filter;
use Mentat::Message::Value::IDEA;

#*******************************************************************************
# CONSTANTS AND GLOBAL VARIABLES DEFINITION SECTION
#*******************************************************************************

#-- Constants -----------------------------------------------------------------#

#-- Static public class variables (our) ---------------------------------------#

#-- Static protected class variables (my) -------------------------------------#

#*******************************************************************************
# INITIALIZATION AND CLEANUP SECTION
#*******************************************************************************

#-- Module initializations ----------------------------------------------------#
BEGIN {
    use vars qw($VERSION $DEVEL @ISA);
    $VERSION = '0.1';
    $DEVEL   = 0;
    @ISA     = ('Mentat::Processor::Engine');
}


#-- Module clean-up code (global destructor) ----------------------------------#
END {

}

#*******************************************************************************
# IMPLEMENTATION SECTION
#*******************************************************************************

=item eh_process($$) [PUBLIC]

  Usage     :
  Purpose   :
  Arguments :
  Returns   :
  Throws    : Dies, if not invoked on object instance or given invalid arguments

=cut

sub eh_process($$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($message,) = @_;
    confess "Message must be a 'Mentat::Processor::Message' implementation" unless blessed($message) and $message->isa('Mentat::Processor::Message');

    # Retrieve the encapsulated IDEA message
    my $alert_message = $message->alert_obj();
    my $alert_id = $alert_message->id();
    my $req_id = $message->id();

    #my $start = time;
    my ($result, $rule, $idx) = $self->{FILTER}->filter($alert_message);
    #my $stop = time;

    # Message denied
    if ($result == 0) {
        $self->_mark_message_deny($message, $rule, $idx, ($stop-$start));
        $self->emit_event('message_commit', $message);
        return EC_STOP;
    }
    # Message allowed
    elsif ($result == 1) {
        $self->_mark_message_allow($message, $rule, $idx, ($stop-$start));
        return (EC_CONTINUE, $message);
    }
}

=item eh_process_multi($$) [PUBLIC]

  Usage     :
  Purpose   :
  Arguments :
  Returns   :
  Throws    : Dies, if not invoked on object instance or given invalid arguments

=cut

sub eh_process_multi($$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($messages, @args) = @_;
    confess "Messages must be given as ARRAY reference" unless $messages and ref $messages eq 'ARRAY';

    Log::Loger->debug(__PACKAGE__, "[ENG:Filter] Filtering ".(scalar @$messages)." messages");

    my $tmp = [];
    my $cntb = 0;
    my $cntp = 0;
    my ($start, $stop, $result, $rule, $idx);
    foreach my $msg (@$messages) {
        eval {
            $start = time;
            ($result, $rule, $idx) = $self->{FILTER}->filter($msg);
            $stop = time;

            # Message denied
            if ($result == 0) {
                Log::Loger->info(__PACKAGE__, "[ENG:Filter] Message '".$msg->id()."' was blocked by filter according to rule '$rule:$idx' (t: ".($stop-$start)." s)");
                $cntb++;
            }
            # Message allowed
            elsif ($result == 1) {
                push(@$tmp, $msg);
                $cntp++;
            }
        };
        if ($@) {
            Log::Loger->error(__PACKAGE__, "[ENG:Filter] Filtering error (skipping message): $@");
        }
    }
    $messages = $tmp;

    Log::Loger->notice(__PACKAGE__, "[ENG:Filter] Filtering complete, '$cntb' messages blocked, '$cntp' messages passed");
    return (EC_CONTINUE, $messages, @args);
}

#-------------------------------------------------------------------------------
# Private interface
#-------------------------------------------------------------------------------

# _init($$;$) [PROTECTED]
#
#  Usage     : return $self->_init(@_);
#  Purpose   : Initialize the new Mentat::Processor::Engine instance
#  Arguments :
#  Returns   :
#  Throws    : Croaks, if invoked on class
#  Comments  : This method must never be called directly, it is supposed to be overloaded by subclasses.

sub _init($$;$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my %config = @_;

    # Call parent version first
    $self = $self->SUPER::_init(%config);

    die "[ENG:Filter] Constructor expects configurations as argument\n" unless %config;
    die "[ENG:Filter] Filter engine configuration 'filter_grammar_file' is missing\n" unless $config{filter_grammar_file};
    die "[ENG:Filter] Filter engine configuration 'filter_schema_file' is missing\n" unless $config{filter_schema_file};
    die "[ENG:Filter] Filter engine configuration 'filter_config' is missing\n" unless $config{filter_config};

    $self->{FILTER} = Data::Filter->new(filter_grammar_file => $config{filter_grammar_file},
                                        filter_schema_file => $config{filter_schema_file},
                                        filter_config => $config{filter_config},
                                        dt_resolver => Mentat::Message::Value::IDEA->new());
    $self->{STATUS} = (defined $config{status})?int($config{status}):200;

    Log::Loger->info(__PACKAGE__, "[ENG:Filter] Using file '$config{filter_grammar_file}' for filter grammar configuration");
    Log::Loger->info(__PACKAGE__, "[ENG:Filter] Using file '$config{filter_schema_file}' for filter schema validation");
    Log::Loger->info(__PACKAGE__, "[ENG:Filter] Using status '$self->{STATUS}' as default response status");

    return $self;
}

# _mark_message_allow($$$$) [PROTECTED]
#
#  Usage     :
#  Purpose   : Mark the message permission
#  Arguments :
#  Returns   :
#  Throws    : Croaks, if invoked on class

sub _mark_message_allow($$$$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($message, $rule, $idx, $duration) = @_;

    my $alert_id = $message->alert_obj->id();

    print "Result: message allowed ($rule:$idx)\n" if $DEVEL; #-+-> DEVEL ONLY <-+-#
    Log::Loger->debug(__PACKAGE__, "[ENG:Filter] Message '$message' containing ALERT with ID '$alert_id' passed the filter via rule '$rule:$idx' (t: $duration s)");

    $self->statistics->{'filter_passed'}++;
    $self->statistics->{'filter_passed_$filter_rule'}++;
}

# _mark_message_deny($$$$) [PROTECTED]
#
#  Usage     :
#  Purpose   : Mark the message denial
#  Arguments :
#  Returns   :
#  Throws    : Croaks, if invoked on class

sub _mark_message_deny($$$$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($message, $rule, $idx, $duration) = @_;

    my $alert_id = $message->alert_obj->id();

    print "Result: message denied ($rule:$idx)\n" if $DEVEL; #-+-> DEVEL ONLY <-+-#
    Log::Loger->info(__PACKAGE__, "[ENG:Filter] Message '$message' containing ALERT with ID '$alert_id' was blocked by filter according to rule '$rule:$idx' (t: $duration s)");

    $self->statistics->{'filter_blocked'}++;
    $self->statistics->{'filter_blocked_$filter_rule'}++;
}

=pod

=back

=cut

1;
