package Mentat::Storage::Factory;
use strict;
use warnings;

#*******************************************************************************
# DOCUMENTATION SECTION
#*******************************************************************************

=head1 NAME

Mentat::Message::Storage::Factory - Class for creating drivers for various IDEA message permanent storages

=head1 SYNOPSIS

 use Mentat::Storage::Factory;

 # Get the desired database driver
 my $storage = Mentat::Storage::Factory->storage($name, %config);

=head1 DESCRIPTION

This factory can be used to retrieve the desired storage engine driver based on
the simple configurations, necessary Perl module is loaded and storage driver is
instantinated and returned.

=head1 AUTHOR

Jan Mach
Cesnet, z.s.p.o
jan.mach@cesnet.cz
http://www.cesnet.cz

=head1 COPYRIGHT

This program is free software; you can redistribute
it and/or modify it under the same terms as Perl itself.

The full text of the license can be found in the
LICENSE file included with this module.

=head1 SEE ALSO

perl(1).

=head1 FUNCTION REFERENCE

=over 4

=cut

#*******************************************************************************
# LIBRARY LOADING SECTION
#*******************************************************************************

#-- Perl core modules ---------------------------------------------------------#
use Carp;
use Scalar::Util 'blessed';

#use Data::Dumper; #-+-> DEVEL ONLY <-+-#
#use Smart::Comments; #-+-> DEVEL ONLY <-+-#

#-- Perl CPAN modules ---------------------------------------------------------#

#-- Custom application modules ------------------------------------------------#
use Mentat::Message::Storage::Mongo;

#*******************************************************************************
# CONSTANTS AND GLOBAL VARIABLES DEFINITION SECTION
#*******************************************************************************

#-- Constants -----------------------------------------------------------------#

#-- Static public class variables (our) ---------------------------------------#

#-- Static protected class variables (my) -------------------------------------#

# List of valid storage engines
my %VALID_ENGINES = (
        'mongo_raw' => 'Mentat::Storage::Mongo',
        'mongo'     => 'Mentat::Message::Storage::Mongo',
    );

#*******************************************************************************
# INITIALIZATION AND CLEANUP SECTION
#*******************************************************************************

#-- Module initializations ----------------------------------------------------#
BEGIN {
    use vars qw($VERSION $DEVEL);
    $VERSION = '0.6';
    $DEVEL   = 0;
}


#-- Module clean-up code (global destructor) ----------------------------------#
END {

}

#*******************************************************************************
# IMPLEMENTATION SECTION
#*******************************************************************************

=item storage() [PUBLIC,STATIC]

  Usage     : my $storage = Mentat::Message::Storage::Factory->new()
  Purpose   : Create and return reference to the new instance
  Arguments : Same as the the _init() method
  Returns   : Mentat::Message::Storage::Factory reference
  Throws    : Croaks, if invoked on object
  Comments  : Internally uses _init() method
  See Also  : _init() method

=cut

sub storage
{
    my $class = shift;
    confess "Class method not invoked on class" if blessed($class);
    my $engine = shift;
    confess "Storage engine name must be given" unless $engine;
    confess "Invalid storage engine '$engine'" unless $VALID_ENGINES{$engine};

    my $engine_class = $VALID_ENGINES{$engine};
    eval "require $engine_class;";
    confess "Error loading engine class '$engine_class': $@" if($@);
    return $engine_class->new(@_);
}

=pod

=back

=cut

1;
